@@ -32,7 +32,7 @@ import Control.Monad
       unless )
 import Crypto.Hash (hashWith, MD5(MD5))
 import Data.Containers.ListUtils (nubOrd)
-import Data.Char (isDigit, isAscii)
+import Data.Char (isDigit, isAscii, isLetter)
 import Data.List (intersperse, (\\))
 import Data.Maybe (catMaybes, fromMaybe, isJust, mapMaybe, isNothing)
 import Data.Monoid (Any (..))
@@ -68,7 +68,6 @@ import Text.Pandoc.Writers.Shared
 import qualified Data.Attoparsec.Text as A
 import qualified Text.Pandoc.UTF8 as UTF8
 import qualified Text.Pandoc.Writers.AnnotatedTable as Ann
-import Data.Char (isLetter)
 import Control.Applicative ((<|>))
 
 -- Work around problems with notes inside emphasis (see #8982)
@@ -184,6 +183,7 @@ pandocToLaTeX options (Pandoc meta blocks) = do
   st <- get
   titleMeta <- escapeCommas <$> -- see #10501
                 stringToLaTeX TextString (stringify $ docTitle meta)
+  subtitleMeta <- stringToLaTeX TextString (stringify $ lookupMetaInlines "subtitle" meta)
   authorsMeta <- mapM (stringToLaTeX TextString . stringify) $ docAuthors meta
   -- The trailer ID is as hash used to identify the PDF. Taking control of its
   -- value is important when aiming for reproducible PDF generation. Setting
@@ -216,7 +216,15 @@ pandocToLaTeX options (Pandoc meta blocks) = do
                            _         -> [])
                     $ lookupMetaInlines "nocite" meta
 
-  let context  =  defField "toc" (writerTableOfContents options) $
+   -- see #7414, avoid escaped underscores
+  let unescapeUnderscore = T.replace "\\_" "_"
+  let bibliography' = map unescapeUnderscore <$>
+                        getField "bibliography" metadata
+
+  let context  =  (case bibliography' of
+                     Nothing -> id
+                     Just xs -> resetField "bibliography" xs) $
+                  defField "toc" (writerTableOfContents options) $
                   defField "lof" (writerListOfFigures options) $
                   defField "lot" (writerListOfTables options) $
                   defField "toc-depth" (tshow
@@ -226,6 +234,7 @@ pandocToLaTeX options (Pandoc meta blocks) = do
                                                  else 0)) $
                   defField "body" main $
                   defField "title-meta" titleMeta $
+                  defField "subtitle-meta" subtitleMeta $
                   defField "author-meta"
                         (T.intercalate "; " authorsMeta) $
                   defField "documentclass" documentClass $
@@ -536,7 +545,7 @@ blockToLaTeX (CodeBlock (identifier,classes,keyvalAttr) str) = do
                Right h -> do
                   when inNote $ modify (\s -> s{ stVerbInNote = True })
                   modify (\s -> s{ stHighlighting = True })
-                  return (flush $ linkAnchor $$ text (T.unpack h))
+                  return (flush $ linkAnchor $$ literal h)
   case () of
      _ | isEnabled Ext_literate_haskell opts && "haskell" `elem` classes &&
          "literate" `elem` classes           -> lhsCodeBlock
@@ -677,7 +686,7 @@ blockToLaTeX (Figure (ident, _, kvs) captnode body) = do
       , stSubfigure = stSubfigure st || isSubfigure
       }
 
-  let containsTable = getAny . (query $ \case
+  let containsTable = getAny . query (\case
         Table {}  -> Any True
         _         -> Any False)
   st <- get
@@ -785,6 +794,7 @@ sectionHeader classes ident level lst = do
       removeInvalidInline x                    = [x]
   let lstNoNotes = foldr (mappend . (\x -> walkM removeInvalidInline x)) mempty lst
   txtNoNotes <- inlineListToLaTeX lstNoNotes
+  txtNoLinksNoNotes <- inlineListToLaTeX (removeLinks lstNoNotes)
   -- footnotes in sections don't work (except for starred variants)
   -- unless you specify an optional argument:
   -- \section[mysec]{mysec\footnote{blah}}
@@ -837,7 +847,7 @@ sectionHeader classes ident level lst = do
                    $$ if unnumbered && not unlisted
                          then "\\addcontentsline{toc}" <>
                                 braces (text sectionType) <>
-                                braces txtNoNotes
+                                braces txtNoLinksNoNotes
                          else empty
 
 -- | Convert list of inline elements to LaTeX.
@@ -947,6 +957,7 @@ inlineToLaTeX (Code (_,classes,kvs) str) = do
   inHeading <- gets stInHeading
   inItem <- gets stInItem
   inSoul <- gets stInSoulCommand
+  inCaption <- gets stInCaption
   let listingsCode = do
         let listingsopts = (case getListingsLanguage classes of
                                 Just l  -> (("language", mbBraced l):)
@@ -999,7 +1010,14 @@ inlineToLaTeX (Code (_,classes,kvs) str) = do
   -- (see #1294). with regular texttt we don't get an error, but we get
   -- incorrect results if there is a space (see #5529).
   let inMbox x = "\\mbox" <> braces x
-  (if inSoul then inMbox else id) <$>
+
+  -- for captions we need to protect VERB with \protect (see #6821)
+  let protect x = "\\protect" <> x
+
+  let optionalProtect = case () of _ | inSoul -> inMbox
+                                     | inCaption -> protect
+                                     | otherwise -> id
+  optionalProtect <$>
    case writerHighlightMethod opts of
      _ | inHeading || inItem  -> rawCode  -- see #5574
      IdiomaticHighlighting    -> listingsCode
@@ -1102,23 +1120,25 @@ inlineToLaTeX (Link (id',_,_) txt (src,_)) =
                      then "\\hyperlink" <> braces (literal lab) <> braces contents
                      else "\\hyperref" <> brackets (literal lab) <> braces contents
      _ -> case txt of
+          -- For soul sommands we need to protect \url and \href in an mbox or
+          -- we get an error (see #9366)
           [Str x] | T.all isAscii x  -- see #8802
                   , unEscapeString (T.unpack x) ==
                     unEscapeString (T.unpack src) ->  -- autolink
                do modify $ \s -> s{ stUrl = True }
                   src' <- stringToLaTeX URLString (escapeURI src)
-                  return $ literal $ "\\url{" <> src' <> "}"
+                  protectInMboxIfInSoul $ literal $ "\\url{" <> src' <> "}"
           [Str x] | Just rest <- T.stripPrefix "mailto:" src,
                     unEscapeString (T.unpack x) == unEscapeString (T.unpack rest) -> -- email autolink
                do modify $ \s -> s{ stUrl = True }
                   src' <- stringToLaTeX URLString (escapeURI src)
                   contents <- inlineListToLaTeX txt
-                  return $ "\\href" <> braces (literal src') <>
+                  protectInMboxIfInSoul $ "\\href" <> braces (literal src') <>
                      braces ("\\nolinkurl" <> braces contents)
           _ -> do contents <- inlineListToLaTeX txt
                   src' <- stringToLaTeX URLString (escapeURI src)
-                  return $ literal ("\\href{" <> src' <> "}{") <>
-                           contents <> char '}')
+                  protectInMboxIfInSoul $ literal ("\\href{" <> src' <> "}{") <>
+                    contents <> char '}')
      >>= (if T.null id'
              then return
              else \x -> do
@@ -1196,9 +1216,12 @@ inlineToLaTeX (Note contents) = do
   let noteContents = nest 2 contents' <> optnl
   beamer <- gets stBeamer
   -- in beamer slides, display footnote from current overlay forward
-  -- and ensure that the note is on the frame, not e.g. the column (#5769)
+  -- and ensure that the note is on the frame, not e.g. the column (#5769, #5954)
+  incremental <- gets stIncremental
   let beamerMark = if beamer
-                      then text "<.->[frame]"
+                      then if incremental
+                           then text "<.->[frame]"
+                           else text "<\\value{beamerpauses}->[frame]"
                       else empty
   if externalNotes
      then do
@@ -1248,6 +1271,15 @@ inSoulCommand pa = do
   modify $ \st -> st{ stInSoulCommand = oldInSoulCommand }
   pure result
 
+-- Inside soul commands some commands need to be protected in an mbox
+-- or we get an error (e.g. see #1294)
+protectInMboxIfInSoul :: (PandocMonad m, HasChars a) => Doc a -> LW m (Doc a)
+protectInMboxIfInSoul command = do
+  inSoul <- gets stInSoulCommand
+  return $ if inSoul
+    then "\\mbox" <> braces command
+    else command
+
 -- Babel languages with a .ldf that works well with all engines (see #8283).
 -- We follow the guidance from the Babel documentation:
 -- "In general, you should do this for European languages written in Latin
