@@ -45,6 +45,7 @@ module Text.Pandoc.Writers.Shared (
                      , toLegacyTable
                      , splitSentences
                      , ensureValidXmlIdentifiers
+                     , removeLinks
                      , setupTranslations
                      , isOrderedListMarker
                      , toTaskListItem
@@ -805,6 +806,14 @@ walkAttr f = walk goInline . walk goBlock
   goBlock (Div attr bs) = Div (f attr) bs
   goBlock x = x
 
+-- | Convert links to spans; most useful when writing elements that must not
+-- contain links, e.g. to avoid nested links.
+removeLinks :: [Inline] -> [Inline]
+removeLinks = walk go
+ where
+  go (Link attr ils _) = Span attr ils
+  go x = x
+
 -- | Set translations based on the `lang` in metadata.
 setupTranslations :: PandocMonad m => Meta -> m ()
 setupTranslations meta = do
